'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _baseError = require('./types/base-error');

var _baseError2 = _interopRequireDefault(_baseError);

var _errorFormatter = require('./error-formatter');

var _errorFormatter2 = _interopRequireDefault(_errorFormatter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 *
 */
var ErrorResponseFormatter = function () {

  /**
   * Create a new ErrorResponseHandler instance.
   */
  function ErrorResponseFormatter() {
    _classCallCheck(this, ErrorResponseFormatter);

    this._formatters = [];
  }

  /**
   * Check if a formatter has been applied.
   *
   * @param {Error} errorType
   * @returns {boolean}
   */


  _createClass(ErrorResponseFormatter, [{
    key: 'hasFormatterForType',
    value: function hasFormatterForType(errorType) {
      var hasFormatter = false;
      this._formatters.forEach(function (f) {
        if (f.formatType() === errorType) {
          hasFormatter = true;
          return;
        }
      });
      return hasFormatter;
    }

    /**
     * Remove a formatter for a specific type of error.
     *
     * @param {Error} errorType
     * @returns {boolean}
     */

  }, {
    key: 'removeForType',
    value: function removeForType(errorType) {
      var formatter = null;
      this._formatters.forEach(function (f) {
        if (f.formatType() === errorType) {
          formatter = f;
          return;
        }
      });

      if (formatter) {
        this._formatters.splice(this._formatters.indexOf(formatter), 1);
      }

      return formatter !== null;
    }

    /**
     * Apply an {ErrorFormatter} to the response handler.
     *
     * @param {ErrorFormatter} errorFormatter
     */

  }, {
    key: 'apply',
    value: function apply(errorFormatter) {
      if (errorFormatter instanceof _errorFormatter2.default) {
        if (this._formatters.indexOf(errorFormatter) > -1) {
          throw new Error('The specified error formatter has already been applied');
        } else {
          if (this.hasFormatterForType(errorFormatter.formatType())) {
            throw new Error('A formatter of the same error type has already been applied');
          } else {
            this._formatters.push(errorFormatter);
          }
        }
      } else {
        throw new Error('{errorFormatter} is not a valid instance of {ErrorFormatter}');
      }
    }

    /**
     * Format a particular type of error.
     *
     * @param {Error} error
     * @returns {object}
     */

  }, {
    key: 'format',
    value: function format(error) {
      if (error instanceof _baseError2.default) {
        delete error.status;
      }

      this._formatters.forEach(function (f) {
        if (error instanceof f.formatType()) {
          error = f.format(error);
        }
      });

      return error;
    }
  }]);

  return ErrorResponseFormatter;
}();

exports.default = ErrorResponseFormatter;
module.exports = exports['default'];