'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _cors = require('cors');

var _cors2 = _interopRequireDefault(_cors);

var _socket = require('socket.io');

var _socket2 = _interopRequireDefault(_socket);

var _helmet = require('helmet');

var _helmet2 = _interopRequireDefault(_helmet);

var _morgan = require('morgan');

var _morgan2 = _interopRequireDefault(_morgan);

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _context = require('./context');

var _context2 = _interopRequireDefault(_context);

var _errors = require('./errors');

var _errors2 = _interopRequireDefault(_errors);

var _qutureConfig = require('quture-config');

var _qutureConfig2 = _interopRequireDefault(_qutureConfig);

var _utilities = require('./utilities');

var _utilities2 = _interopRequireDefault(_utilities);

var _serveFavicon = require('serve-favicon');

var _serveFavicon2 = _interopRequireDefault(_serveFavicon);

var _bodyParser = require('body-parser');

var _bodyParser2 = _interopRequireDefault(_bodyParser);

var _randomstring = require('randomstring');

var _randomstring2 = _interopRequireDefault(_randomstring);

var _cookieParser = require('cookie-parser');

var _cookieParser2 = _interopRequireDefault(_cookieParser);

var _errorCode = require('./errors/error-code');

var _errorCode2 = _interopRequireDefault(_errorCode);

var _methodOverride = require('method-override');

var _methodOverride2 = _interopRequireDefault(_methodOverride);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _errorFormatter = require('./errors/error-formatter');

var _errorFormatter2 = _interopRequireDefault(_errorFormatter);

var _errorResponseFormatter = require('./errors/error-response-formatter');

var _errorResponseFormatter2 = _interopRequireDefault(_errorResponseFormatter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var debug = (0, _debug2.default)('quture:app');

/**
 *
 */

var Quture = function () {

  /**
   * Create a new {Quture} instance.
   *
   * @param {object} options
   */
  function Quture(options) {
    _classCallCheck(this, Quture);

    options = options || {};

    if (!options.rootDir) {
      throw new Error('{options.rootDir} is required');
    } else {
      if (_fs2.default.lstatSync(options.rootDir).isDirectory()) {
        this._rootDir = options.rootDir;
      } else {
        throw new Error('{options.rootDir} does not exist');
      }
    }

    /**
     * Configure the configuration handler
     */
    if (!options.config) {
      if (options.configDir && typeof options.configDir !== 'string') {
        throw new Error('{options.configDir} must be of type string');
      }

      var configDir = options.configDir || _path2.default.join(this._rootDir, 'config');

      _utilities2.default.ensureDirSync(configDir);

      options.config = new _qutureConfig2.default({ configDir: configDir });
    } else {
      if (!(options.config instanceof _qutureConfig2.default)) {
        throw new Error('{options.config} must be a valid instance of {Config}');
      }
    }

    this._config = options.config;

    var appConfigFile = _path2.default.join(__dirname, 'config', 'app.js');

    if (!this.isConfigurationFilePublished(this._config, appConfigFile)) {
      this.publishConfigurationFile(this._config, appConfigFile);
    }

    /**
     * Setup views directory
     */
    if (options.viewsDir) {
      this._viewsDir = options.viewsDir;
    } else {
      this._viewsDir = _path2.default.join(this.getRootDirectory(), 'views');
    }

    _utilities2.default.ensureDirSync(this._viewsDir);

    /**
     * Setup public directory
     */
    if (options.publicDir) {
      this._publicDir = options.publicDir;
    } else {
      this._publicDir = _path2.default.join(this.getRootDirectory(), 'public');
    }

    _utilities2.default.ensureDirSync(this._publicDir);

    /**
     * Setup storage directory
     */
    if (options.storageDir) {
      this._storageDir = options.storageDir;
    } else {
      this._storageDir = _path2.default.join(this.getRootDirectory(), 'storage');
    }

    _utilities2.default.ensureDirSync(this._storageDir);

    /**
     * Setup Websockets
     */
    this._io = (0, _socket2.default)();

    /**
     * Setup Error Response Formatter
     */
    this._errorResponseFormatter = new _errorResponseFormatter2.default();

    /**
     * Contextualize the instance as a global context.
     */
    Quture.contextualize(this, 'global');

    /**
     * Create a new router
     */
    this._router = Quture.createRouter();

    /**
     * //
     */
    this._isBootstrapped = false;
  }

  /**
   * Retrieve the utilities helper class.
   *
   * @return {Utilities}
   */


  _createClass(Quture, [{
    key: 'getConfig',


    /**
     * Retrieve the configuration instance.
     *
     * @param {string} namespace
     * @returns {Config}
     * @throws {Error}
     */
    value: function getConfig(namespace) {
      if (namespace) {
        if (typeof namespace === 'string') {
          return this._pluginManager.getPluginByName(namespace).getConfig();
        } else {
          throw new Error('{namespace} parameter must be of type string and reference a plugin');
        }
      }
      return this._config;
    }

    /**
     * Check if the specified configuration file is already
     * published to the config folder.
     *
     * @param {Config} config
     * @param {string} fileName
     * @returns {boolean}
     */

  }, {
    key: 'isConfigurationFilePublished',
    value: function isConfigurationFilePublished(config, fileName) {
      if (config instanceof _qutureConfig2.default) {
        return _utilities2.default.isFilePublished(fileName, config.getConfigDirectory());
      } else {
        throw new Error('{config} must be an instance of {Config}');
      }
    }

    /**
     * Publish the configuration file with the specified name
     * into the specified config folder.
     *
     * @param {Config} config
     * @param {string} filePath
     */

  }, {
    key: 'publishConfigurationFile',
    value: function publishConfigurationFile(config, filePath) {
      if (config instanceof _qutureConfig2.default) {
        return _utilities2.default.publishFile(filePath, config.getConfigDirectory());
      } else {
        throw new Error('{config} must be an instance of {Config}');
      }
    }

    /**
     * Retrieve the root directory.
     *
     * @returns {string}
     */

  }, {
    key: 'getRootDirectory',
    value: function getRootDirectory() {
      return this._rootDir;
    }

    /**
     * Retrieve the path to the views directory.
     *
     * @returns {string}
     */

  }, {
    key: 'getViewsDirectory',
    value: function getViewsDirectory() {
      return this._viewsDir;
    }

    /**
     * Retrieve the path to the public directory.
     *
     * @returns {string}
     */

  }, {
    key: 'getPublicDirectory',
    value: function getPublicDirectory() {
      return this._publicDir;
    }

    /**
     * Retrieve the path to the storage directory.
     *
     * @returns {string}
     */

  }, {
    key: 'getStorageDirectory',
    value: function getStorageDirectory() {
      return this._storageDir;
    }

    /**
     * Retrieve the socket.io instance.
     *
     * @returns {Socket}
     */

  }, {
    key: 'getWebSocketHandle',
    value: function getWebSocketHandle() {
      return this._io;
    }

    /**
     * Retrieve the main {Express.Router} instance.
     *
     * @returns {Express.Router}
     */

  }, {
    key: 'getRouter',
    value: function getRouter() {
      return this._router;
    }

    /**
     * Create a router with a context applied to each request.
     *
     * @param {object} options
     * @return {Express.Router}
     */

  }, {
    key: 'getErrorResponseFormatter',


    /**
     * Retrieve the {ErrorResponseFormatter} instance.
     *
     * @returns {ErrorResponseFormatter}
     */
    value: function getErrorResponseFormatter() {
      return this._errorResponseFormatter;
    }

    /**
     * Create a new {ErrorFormatter} instance.
     *
     * @returns {ErrorFormatter}
     */

  }, {
    key: 'bootstrap',


    /**
     * Bootstrap the application.
     *
     * @returns {Express}
     */
    value: function bootstrap() {
      var self = this;

      if (!this._isBootstrapped) {
        self._isBootstrapped = true;

        self._app = (0, _express2.default)();
        self._app.disable('etag');

        /**
         * Configure the application proxy settings
         */
        self._app.set('trust proxy', self.getConfig().get('app.behindProxy', false));

        /**
         * Configure the application port
         */
        self._app.set('port', function (val) {
          var port = parseInt(val, 10);
          if (isNaN(port)) {
            return val;
          }
          if (port >= 0) {
            return port;
          }
          return false;
        }(self.getConfig().get('app.port', 80)));

        /**
         * Setup view engine && pretty responses
         */
        if (self.getConfig().get('app.prettify', false)) {
          self._app.locals.pretty = true;
          self._app.set('json spaces', 2);
        }

        self._app.set('views', self.getViewsDirectory());
        self._app.set('view engine', self.getConfig().get('app.viewEngine', 'pug'));

        /**
         * Serve the favicon
         */
        var faviconPath = _path2.default.join(self.getPublicDirectory(), 'favicon.ico');
        if (_fs2.default.existsSync(faviconPath)) {
          self._app.use((0, _serveFavicon2.default)(faviconPath));
        }

        /**
         * ... Middleware ...
         */
        if (process.env.NODE_ENV !== 'testing') {
          self._app.use((0, _morgan2.default)('dev'));
        }

        self._app.use(_bodyParser2.default.json());
        self._app.use(_bodyParser2.default.urlencoded({ 'extended': true }));
        self._app.use((0, _cookieParser2.default)(self.getConfig().get('app.key', null), {
          'httpOnly': true,
          'secure': self.getConfig().get('app.secure', false)
        }));
        self._app.use((0, _helmet2.default)());
        self._app.use((0, _methodOverride2.default)('X-HTTP-Method-Override'));
        self._app.use((0, _cors2.default)());

        // Append a method to retrieve the global context
        self._app.use(function (request, response, next) {
          request.quture = self;
          self.generateContext(request);

          /**
           * Retrieve the global context.
           *
           * @return {Context}
           */
          request.getGlobalContext = function () {
            return self.getContext(request);
          };

          next();
        });

        /**
         * Configure non-api endpoints
         */
        self._app.use('/public', _express2.default.static(this.getPublicDirectory()));

        // Apply router after the middleware has been applied
        self._app.use(self._router);

        /**
         * Redirect other calls to the application router, if a route cannot be found
         * or the error isn't handled, then fallback to the public directory thus
         * leaving the client to deal with it.
         */
        self._app.use(function (request, response, next) {
          if (request.originalUrl.startsWith('/api')) {
            next(new _errors2.default.NotFoundError());
          } else {
            var indexFile = _path2.default.join(self.getPublicDirectory(), 'index.html');
            if (_fs2.default.existsSync(indexFile)) {
              response.sendFile(indexFile);
            } else {
              next();
            }
          }
        }, _express2.default.static(self.getPublicDirectory()));

        // Apply 404 catch
        self._app.use(function (request, response, next) {
          next(new _errors2.default.NotFoundError()); // Resource not found
        });

        // Apply a route response handler
        self._app.use(function (error, request, response, next) {
          debug(error);
          response.status(error.status || 200).json({
            errors: [self.getErrorResponseFormatter().format(error)]
          });
        });
      }

      return self._app;
    }
  }], [{
    key: 'getUtilities',
    value: function getUtilities() {
      return _utilities2.default;
    }
  }, {
    key: 'createRouter',
    value: function createRouter(options) {
      var router = _express2.default.Router();
      Quture.contextualize(router);

      router.use(function (request, response, next) {
        router.generateContext(request);
        next();
      });

      return router;
    }

    /**
     * Retrieve the base error types.
     *
     * @returns {object}
     */

  }, {
    key: 'createErrorFormatter',
    value: function createErrorFormatter(errorType, errorFormatter) {
      return new _errorFormatter2.default(errorType, errorFormatter);
    }

    /**
     * Create a new {ErrorCode} instance.
     *
     * @returns {ErrorCode}
     */

  }, {
    key: 'createErrorCode',
    value: function createErrorCode(code, meta) {
      return new _errorCode2.default(code, meta);
    }

    /**
     * Apply a context to the specified object.
     *
     * @param {object} object
     */

  }, {
    key: 'contextualize',
    value: function contextualize(object, namespace) {
      if (namespace) {
        if (typeof namespace === 'string') {
          object._ctxId = namespace;
        } else {
          throw new Error('{namespace} must be of type {string}');
        }
      } else {
        object._ctxId = _randomstring2.default.generate(5);
      }

      /**
       * Retrieve context ID.
       *
       * @returns {string}
       */
      object.getContextId = function () {
        return object._ctxId;
      };

      /**
       * Instantiate a new Context and apply it to the specified object.
       *
       * @param {object} k
       */
      object.generateContext = function (k) {
        if (object.getContext(k)) {
          throw new Error('A context already exists for the specified object');
        } else {
          var context = new _context2.default();
          k['q-ctx-' + object.getContextId()] = context;
          return context;
        }
      };

      /**
       * Retrieve context from the specified object.
       *
       * @returns {Context}
       */
      object.getContext = function (k) {
        return k['q-ctx-' + object.getContextId()];
      };
    }
  }, {
    key: 'ErrorTypes',
    get: function get() {
      return _errors2.default;
    }
  }]);

  return Quture;
}();

exports.default = Quture;
module.exports = exports['default'];