'use strict';

import fs from 'fs';
import path from 'path';
import fsx from 'fs-extra';

export default class Utilities {

  /**
   * Ensure the directory is created.
   *
   * @param {string} dirPath
   */
  static ensureDirSync(dirPath) {
    if (!fs.existsSync(dirPath)) {
      fsx.ensureDirSync(dirPath);
    } else {
      if (!fs.lstatSync(dirPath).isDirectory()) {
        throw new Error('Unable to create a directory at [' + dirPath + '] because a file with the name [' + path.parse(dirPath).base + '] already exists');
      }
    }
  }

  /**
   * Check if the specified file is already
   * published to the specified directory.
   *
   * @param {string} fileName
   * @param {string} dirPath
   * @returns {boolean}
   */
  static isFilePublished(fileName, dirPath) {
    let filePath = path.join(dirPath, fileName);

    if (fs.existsSync(filePath) && fs.lstatSync(filePath).isFile()) {
      return true;
    } else {
      return false;
    }
  }

  /**
   * Publish the specified file with the specified name
   * into the specified directory.
   *
   * @param {string} filePath
   * @param {string} dirPath
   */
  static publishFile(filePath, dirPath) {
    let fileName = path.parse(filePath).base;

    if (!Utilities.isFilePublished(fileName, dirPath)) {
      if (!fs.lstatSync(filePath).isDirectory()) {
        fsx.copySync(filePath, path.join(dirPath, fileName));
      }
    }
  }

}
